@@ -10,37 +10,23 @@
 #' Approximate errors can be used by setting \code{true_error = FALSE} when using \code{\link{plot.calibration_simplex}}.
 #'
 #' @references Daniel S. Wilks, 2013, The Calibration Simplex: A Generalization of the Reliability Diagram for Three-Category Probability Forecasts, \emph{Weather and Forecasting}, \strong{28}, 1210-1218
+#' @references Resin, J. (2020), A Simple Algorithm for Exact Multinomial Tests, \emph{Preprint} \url{https://arxiv.org/abs/2008.12682}
+#' 
 #' @seealso \code{\link{plot.calibration_simplex}}
 #' @seealso \code{\link{ternary_forecast_example}}
 #'
 #' @importFrom stats aggregate
+#' @importFrom ExactMultinom multinom_test_cpp
 
 calibration_simplex.default = function(n = 10,
                                        p1 = NULL,
                                        p2 = NULL,
                                        p3 = NULL,
                                        obs = NULL,
-                                       percentagewise = FALSE,
-                                       p_a = NULL,
-                                       p_n = NULL,
-                                       p_b = NULL) {
+                                       test_stat = "LLR",
+                                       percentagewise = FALSE) {
   factor_percent = if(percentagewise) 100 else 1 #=div (prev)
 
-  if(!is.null(p_a)){
-    warning("The parameter p_a is deprecated and will be removed. Please use p3 instead!")
-    if(is.null(p3)) p3 = p_a
-  }
-  if(!is.null(p_b)){
-    warning("The parameter p_b is deprecated and will be removed. Please use p1 instead!")
-    if(is.null(p1)) p1 = p_b
-  }
-  if(!is.null(p_n)){
-    warning("The parameter p_n is deprecated and will be removed. Please use p2 instead!")
-    if(is.null(p2)) p2 = p_n
-  }
-
-
-
   if(is.null(obs)) stop("Observations are missing!")
   stopifnot(all(obs %in% c(1,2,3)))
 
@@ -63,6 +49,7 @@ calibration_simplex.default = function(n = 10,
   else if(is.null(p2)) {
     if(any(p3 < 0)||any(p1 < 0)) stop("Negative probabilities detected!")
     if(any(p3+p1>(1 + eps)*factor_percent)) stop("Specified probabilities do not sum to <=1!")
+    p2 = factor_percent-p1-p3
   }
 
   else {
@@ -85,32 +72,66 @@ calibration_simplex.default = function(n = 10,
     return(bin)
   }
 
-  p3 = p3/factor_percent
   p1 = p1/factor_percent
+  p2 = p2/factor_percent
+  p3 = p3/factor_percent
 
   bin = mapply(assign_bin,p1,p3)
 
-  data = data.frame(p3,p1,obs,bin)
+  data = data.frame(p1,p2,p3,obs,bin)
 
   out = list(n = n,
              n_bins = n_bins,
              n_obs = n_obs,
              freq = rep(0,n_bins),
-             cond_rel_freq_1 = rep(NA,n_bins),
-             cond_rel_freq_3 = rep(NA,n_bins),
-             cond_p3_ave = rep(NA,n_bins),
-             cond_p1_ave = rep(NA,n_bins))
 
-  cond_rel_freq = prop.table(table(data[,3:4]),2)
-  cond_p_ave = aggregate(data[,1:2],list(data[,4]),mean)
+             cond_rel_freq = matrix(rep(NA,3*n_bins),ncol = 3),
+             cond_ave_prob = matrix(rep(NA,3*n_bins),ncol = 3),
+             # cond_rel_freq_1 = rep(NA,n_bins),
+             # cond_rel_freq_3 = rep(NA,n_bins),
+             # cond_p3_ave = rep(NA,n_bins),
+             # cond_p1_ave = rep(NA,n_bins),
+
+             pvals = rep(NA,n_bins))
+
+  cond_rel_freq = as.matrix(prop.table(table(rbind(data[,4:5],c(1,0),c(2,0),c(3,0))),2)[,-1]) #fixes error, when obs does not contain all three outcomes
+                                                                                              # as.matrix() allows for a single bin
+  #cond_rel_freq = prop.table(table(data[,3:4]),2) # replaced in 0.4.0
+  cond_p_ave = aggregate(data[,1:3],list(data[,5]),mean)
 
   bins = cond_p_ave[,1]
 
-  out$freq[bins] = margin.table(table(data[,4]),1)
-  out$cond_rel_freq_1[bins] = cond_rel_freq[1,]
-  out$cond_rel_freq_3[bins] = cond_rel_freq[3,]
-  out$cond_p3_ave[bins] = cond_p_ave[,2]
-  out$cond_p1_ave[bins] = cond_p_ave[,3]
+  out$freq[bins] = margin.table(table(data[,5]),1)
+
+  out$cond_rel_freq[bins,] = t(cond_rel_freq)
+  # out$cond_rel_freq_1[bins] = cond_rel_freq[1,]
+  # out$cond_rel_freq_3[bins] = cond_rel_freq[3,]
+
+  out$cond_ave_prob[bins,] = as.matrix(cond_p_ave[,2:4])
+  #out$cond_ave_prob[bins,2] = 1 - out$cond_ave_prob[bins,1] - out$cond_ave_prob[bins,3]
+  # out$cond_p3_ave[bins] = cond_p_ave[,2]
+  # out$cond_p1_ave[bins] = cond_p_ave[,3]
+
+  # Calculate pvalues
+
+  stat = which(c("Prob","Chisq","LLR") == test_stat)
+  if(!length(stat) == 0){
+    for(bin in 1:n_bins){
+      if(out$freq[bin] > 0){
+        x = out$cond_rel_freq[bin,]*out$freq[bin]
+        p = out$cond_ave_prob[bin,]
+        if(all(p > 0)) out$pvals[bin] = multinom_test_cpp(x,p)[stat]
+        else if(sum(p>0) == 2){
+          if(x[!(p>0)] == 0) out$pvals[bin] = multinom_test_cpp(x[p>0],p[p>0])[stat]
+          else out$pvals[bin] = -1
+        }
+        else if(sum(p>0) == 1){
+          if(all(x[!(p>0)] == 0)) out$pvals[bin] = 1
+          else out$pvals[bin] = -1
+        }
+      }
+    }
+  }
 
   class(out) = append(class(out),"calibration_simplex")
   return(out)
